
PROGRAM rtofprep

!    Name: rtofprep
!    Subroutines: readrtfinp
!                 outncf
!
!    Author: Biju Thomas on Jan 12, 2016 (GSO-URI)
!
!  1. Purpose :
!
! Read GLOBAL RTOF data (binary) and inerpolate to MPIPOM grids 
! Pepare initial conditions for MPIPOM-TC.
!


USE varsmd
USE outncfmd
USE setupmd
USE fillupmd
USE smooth2d_md
USE readrtfinpmd
IMPLICIT NONE

  INTEGER :: ionedim, allo_err
  INTEGER :: i, j, k, nflg, igfssst
  INTEGER, PARAMETER :: maskval = 0
  INTEGER, DIMENSION(:,:), ALLOCATABLE :: mask
  REAL :: blon, blat, whm
  REAL, PARAMETER :: d2r = 3.1416/180., rearth = 6371.e3
  REAL, PARAMETER :: xwidth = 83.2, ywidth = 37.5, hmax = 5500.0
  REAL, PARAMETER :: largetval = 1.2676506E+30, largewval=1.2676506E+30
  REAL, DIMENSION(:,:), ALLOCATABLE :: f2d
  CHARACTER(LEN=4) :: odata

  INTEGER :: im1, jm1
  INTEGER :: n
  REAL :: clon, clat, dlonlat, dvkm, dvr, vitR
  INTEGER :: nvit
  integer vhour, vlat, vlon, vmx, vrmw
  integer readunit
  integer*4 vdate
  integer vni,vn1,vipmax
  integer vday,vmonth,vyear,end_flag
  integer vgarb(5),vRd1(4),vRd2(4)
  character*19 vname

  integer pln
  parameter(PLN=100)
  real,DIMENSION(PLN)   :: vitX,vitY
  real,DIMENSION(PLN)   :: vTM,vPRES,vPRES0,vRMAXa,vWSPMAX,vRADCLS
  real,DIMENSION(PLN,4) :: vR18v,vR26v


  READ(5,*) odata ! Region-specific information
  READ(5,*) ionedim
  READ(5,*) igfssst
  READ(5,*) runid
!  READ(5,*) nvit

  WRITE(6,*) odata
  CALL swcorner(odata, blon, blat)
  WRITE(6,*) 'blon,blat=',blon, blat
  CALL modeldf()

! MPIPOM-TC full (z) and half (zz) sigma levels
  DO k = 1,kb
    z(k) = -z1(k)/z1(kb)
  END DO
  DO k = 1,kb-1
    zz(k) = 0.5*(z(k)+z(k+1))
  END DO
  zz(kb) = z(kb)

  DO j = 1,jm
    DO i = 1,im
      east_e(i,j) = blon+(i-1)*xwidth/(im-1)
      north_e(i,j) = blat+(j-1)*ywidth/(jm-1)
    END DO
  END DO

! MPIPOM-TC grid spacing

  DO j = 1,jm
    DO i = 2,im-1
      dx(i,j) = 0.5*d2r*rearth*sqrt(((east_e(i+1,j)-east_e(i-1,j))* &
        cos(north_e(i,j)*d2r))**2.+(north_e(i+1,j)-north_e(i-1,j))**2.)
    END DO
    dx(1,j) = dx(2,j)
    dx(im,j) = dx(im-1,j)
  END DO

  DO i = 1,im
    DO j = 2,jm-1
      dy(i,j) = 0.5*d2r*rearth*sqrt(((east_e(i,j+1)-east_e(i,j-1))* &
        cos(north_e(i,j)*d2r))**2.+(north_e(i,j+1)-north_e(i,j-1))**2.)
    END DO
    dy(i,1) = dy(i,2)
    dy(i,jm) = dy(i,jm-1)
  END DO

! MPIPOM-TC cell corners

  DO j = 2,jm
    DO i = 2,im
      east_c(i,j) = 0.25*(east_e(i,j)+east_e(i,j-1)+ &
                         east_e(i-1,j)+east_e(i-1,j-1))
      north_c(i,j) = 0.25*(north_e(i,j)+north_e(i,j-1)+ &
                          north_e(i-1,j)+north_e(i-1,j-1))
    END DO
  END DO
  DO i = 2,im
    east_c(i,1) = 2.*east_c(i,2)-east_c(i,3)
    north_c(i,1) = 2.*north_c(i,2)-north_c(i,3)
  END DO
  east_c(1,1) = 2.*east_c(2,1)-east_c(3,1)
  DO j = 2,jm
    east_c(1,j) = 2.*east_c(2,j)-east_c(3,j)
    north_c(1,j) = 2.*north_c(2,j)-north_c(3,j)
  END DO
  north_c(1,1) = 2.*north_c(1,2)-north_c(1,3)

  IF (ionedim.eq.1) THEN

! MPIPOM-TC u points

    DO i = 1,im
      DO j = 1,jm
        east_u(i,j) = east_e(i,j)
        north_u(i,j) = north_e(i,j)
      END DO
    END DO

! MPIPOM-TC v points

    DO j = 1,jm
      DO i = 1,im
        east_v(i,j) = east_e(i,j)
        north_v(i,j) = north_e(i,j)
      END DO
    END DO

  ELSE

! MPIPOM-TC u points

    DO i = 1,im
      DO j = 1,jm-1
        east_u(i,j) = 0.5*(east_c(i,j)+east_c(i,j+1))
        north_u(i,j) = 0.5*(north_c(i,j)+north_c(i,j+1))
       END DO
      east_u(i,jm) = 0.5*(east_c(i,jm)*3.-east_c(i,jm-1))
      north_u(i,jm) = 0.5*(north_c(i,jm)*3.-north_c(i,jm-1))
    END DO

! MPIPOM-TC v points

    DO j = 1,jm
      DO i = 1,im-1
        east_v(i,j) = 0.5*(east_c(i,j)+east_c(i+1,j))
        north_v(i,j) = 0.5*(north_c(i,j)+north_c(i+1,j))
      END DO
      east_v(im,j) = 0.5*(east_c(im,j)*3.-east_c(im-1,j))
      north_v(im,j) = 0.5*(north_c(im,j)*3.-north_c(im-1,j))
    END DO

  END IF

! MPIPOM-TC rotation angle

  DO j = 1,jm
    DO i = 1,im-1
      rot(i,j) = atan2(north_e(i+1,j)-north_e(i,j), &
                       east_e(i+1,j)-east_e(i,j))
    END DO
    rot(im,j) = rot(im-1,j)
  END DO

  READ(5,*) clon
  READ(5,*) clat
  print*, 'clon, clat=', clon, clat
  if( clon-east_e(1,1) >= 360. ) clon=clon-360.
  if( clon-east_e(1,1) <= -360. ) clon=clon+360.
  print*, 'After adjustment if needed, clon, clat=', clon, clat
  
  vipmax=0
!  IF (nvit .GT. 0 ) THEN
    readunit = 15
    !---------------------- Reading message file --------------------
17  format(A19,I6,1X,I4,1X,I3,1X,I4,1X,I3,1X,I3,3I5, &
       1X,I2,1X,I3,1X,I4,1X,I4,1X,I4,1X,I4,1X,I4,1X,I4,1X,I4,1X,I4)
    open(readunit,file="track",status='old')
    end_flag=0.
    I=0
    do while(end_flag.eq.0)
      read(readunit,17) vname,vdate,vhour,vlat,vlon,vgarb,vmx,vrmw,vrd1,vrd2
      write(*,17) vname,vdate,vhour,vlat,vlon,vgarb,vmx,vrmw,vrd1,vrd2
      if(vdate.eq.0) then
        end_flag = 1
      else
        I=I+1
        vdate=vdate*100+vhour/100
        vTM(i)=float(vdate)
        vitX(i)=vlon/10.                       !RMY: Eastern Hemsiphere
        vitY(i)=vlat/10.                       !RMY: Northern Hemisphere
        if (east_e(1,1).lt.0.) vitX(i)=-vitX(i)  !RMY: Western Hemisphere
        if (north_e(1,1).lt.0.) vitY(i)=-vitY(i) !RMY: Southern Hemisphere
        vPRES(i)=float(vgarb(3))
        vPRES0(i)=float(vgarb(4))
        vRADCLS(i)=float(vgarb(5))*1000
        vWSPMAX(i)=float(vmx)
        vRMAXa(i)=float(vrmw)
        do vni=1,4
          vn1=vni+(1-mod(vni,2))*sign(2,3-vni)
          vR18v(i,vni)=vRd1(vn1)
          vR26v(i,vni)=vRd2(vn1)
          if(vwspmax(i).le.26.or.vR26v(i,vni).le.vRMAXa(i)) &
             vR26v(i,vni)=-999
          if(vwspmax(i).le.18.or.vR18v(i,vni).le.vRMAXa(i)) &
             vR18v(i,vni)=-999
          if(vR26v(i,vni).gt.vR18v(i,vni)) vR26v(i,vni)=-999
        end do
      end if
    end do
    vipmax=I
    close(readunit)
    if (vipmax>0) then
      write(*,*) 'Storm track info after conversion:' 
      write(*,*) 'vTM(i),vitX(i),vitY(i),vPRES(i),vPRES0(i),vRADCLS(i),vWSPMAX(i),vRMAXa(i),vR18v(i,:),vR26v(i,:)' 
      do i=1,vipmax
        write(*,'(16F15.5)') vTM(i),vitX(i),vitY(i),vPRES(i),vPRES0(i),vRADCLS(i),vWSPMAX(i),vRMAXa(i),vR18v(i,:),vR26v(i,:)  
      end do
    else
      write(*,*) 'No storm track info available' 
    endif
!  ENDIF

  IF (blon < 0.0 .AND. (odata == 'rttr' .OR. odata == 'rtep' .OR. &
      odata == 'rtse' .OR. odata == 'rtsa') ) THEN
     whm = 360.0
  ELSE
     whm = 0.0
  ENDIF 

  CALL readrtfinp(im, jm, nl, east_e+whm, north_e,east_u+whm, north_u,     &
                  east_v+whm, north_v, zrtof, t, s, uin, vin, h)
 
  ALLOCATE(mask(im,jm), STAT=allo_err)
  DO j = 1,jm
    DO i = 1,im
      IF ( h(i,j) == largetval ) THEN
         mask(i,j) = maskval
      ELSE
         mask(i,j) = 1
      ENDIF
    ENDDO
  ENDDO
  CALL  smooth2d9ptwave(maskval,mask, h, im, jm)
  CALL  smooth2d9ptwave(maskval,mask, h, im, jm)
  CALL  smooth2d9ptwave(maskval,mask, h, im, jm)
  DEALLOCATE(mask)

!  IF ( .NOT. ALLOCATED(f2d)) ALLOCATE(f2d(im,jm))
!  f2d = h
!  DO j = 1, jm
!    DO i = 1, im
!      DO k = 2, nl
!        IF( zrtof(k) <=  h(i,j) .AND. h(i,j) /= largetval) THEN
!          IF ( t(i,j,k) == largetval .OR. s(i,j,k) == largetval ) THEN
!            f2d(i,j) = zrtof(k-1)
!            EXIT
!          ENDIF
!        ENDIF
!      ENDDO
!    ENDDO
!  ENDDO
!  h = f2d
!  DEALLOCATE(f2d)

  CALL fillup(largetval, zrtof, h, t, s)
  DO j = 1, jm
    DO i = 1, im
       DO k = 1, nl
         IF( zrtof(k) >  h(i,j) ) THEN
            t(i,j,k) = largetval
            s(i,j,k) = largetval
            uin(i,j,k) = largetval
            vin(i,j,k) = largetval
         ENDIF
       ENDDO
    ENDDO
  ENDDO

! Set uin, vin, and el to 0

  DO k = 1,nl
    DO j = 1,jm
      DO i = 1,im
        uin(i,j,k) = 0. ! calculated as geostrophic in old POM-TC
        vin(i,j,k) = 0. ! calculated as geostrophic in old POM-TC
      END DO
    END DO
  END DO

  DO j = 1,jm
    DO i = 1,im
      el(i,j) = 0.
    END DO
  END DO
 
! Neutral profile generation
  DO j = 1,jm
    DO i = 1,im
      nflg = 0
      DO k = 2,nl
        IF (t(i,j,k) == largetval .OR. s(i,j,k) ==  largetval .OR. nflg .EQ. 1) THEN
          t(i,j,k) = t(i,j,k-1)
          s(i,j,k) = s(i,j,k-1)
          nflg = 1
        END IF
      END DO
    END DO
  END DO


! MPIPOM-TC land/sea mask

  DO j = 1,jm
    DO i = 1,im
      IF (h(i,j) < 11.) h(i,j) = 1.  ! Restrict shallowest topography to 11-m depth
      IF (h(i,j) > 1. .AND. h(i,j) /= largetval) THEN
        h(i,j) = MIN(h(i,j), hmax)  ! Restrict bottom topography to 5500-m depth
        fsm(i,j) = 1.
        dum(i,j) = 1.
        dvm(i,j) = 1.
      ELSE
        fsm(i,j) = 0.
        dum(i,j) = 0.
        dvm(i,j) = 0.
        h(i,j) = 1.
      END IF
    END DO
  END DO


  IF (odata == 'rttr') THEN   ! Pacific removal
    DO j = 1,85
      DO i = 1,69
        fsm(i,j) = 0.
        dum(i,j) = 0.
        dvm(i,j) = 0.
        h(i,j) = 1.
      END DO
    END DO
    DO j = 1,55
      DO i = 70,142
        fsm(i,j) = 0.
        dum(i,j) = 0.
        dvm(i,j) = 0.
        h(i,j) = 1.
      END DO
    END DO
  END IF

  IF (odata == 'rtep') THEN   ! Caribbean and Gulf of Mexico removal
    DO j = 120,jm
    DO i = 810,im
      fsm(i,j) = 0.
      dum(i,j) = 0.
      dvm(i,j) = 0.
      h(i,j) = 1.
    END DO
    END DO

    DO j = 145,jm
    DO i = 715,im
      fsm(i,j) = 0.
      dum(i,j) = 0.
      dvm(i,j) = 0.
      h(i,j) = 1.
    END DO
    END DO
  END IF

  IF (odata == 'rtsa') THEN   ! Southeast Pacific removal
    DO j = 1,jm
      DO i = 1,194
        fsm(i,j) = 0.
        dum(i,j) = 0.
        dvm(i,j) = 0.
        h(i,j) = 1.
      END DO
    END DO
  END IF

  IF (ionedim.ne.1) THEN
! MPIPOM-TC land/sea mask modification
    DO j = 1,jm-1
      DO i = 1,im
        IF (fsm(i,j) == 0. .AND. fsm(i,j+1) /= 0.) dvm(i,j+1) = 0.
      END DO
    END DO

    DO j = 1,jm
      DO i = 1,im-1
        IF (fsm(i,j) == 0. .AND. fsm(i+1,j) /= 0.) dum(i+1,j) = 0. 
      END DO
    END DO
  END IF

! Read and assimilate gfs sst into temperature field

 WRITE(6,*) 'SST assimilation (0=no; 1=yes)? ',igfssst
 IF (igfssst.eq.1) THEN
     print*, 'Preparing to read gfs sst...'
     read(23,224) im1,jm1 ! ensure lonlat.gfs   is linked to fort.23
     rewind(23)           ! ensure sst.gfs.dat  is linked to fort.21
 224 format(1x,2i5)       ! ensure mask.gfs.dat is linked to fort.22
     call serftempr(im1,jm1,stm,fsm,east_e,north_e,im,jm)
     print*, '... serftempr read gfs sst: im1,jm = ',im1,jm1

     tmix=t
     told=t

     print*, 'Preparing to assimilate gfs sst...'
!    call mixsstz(t,stm,h,fsm,zrtof,im,jm,nl,0)
     call mixsstz(tmix,stm,h,fsm,zrtof,im,jm,nl,0)
     print*, '... mixsstz assimilated gfs sst'

!  IF ( nvit > 0 .AND. vipmax > 0 ) THEN
  IF ( vipmax > 12 ) THEN
    ! R > 6*vitR:          Assimilate GFS SST 
    ! 4*vitR < R < 6*vitR: Merge GFS SST and RTOFS T
    ! R < 4*vitR:          Use RTOFS T
    print*, 'Track file exists, use RTOFS SST along the track'
    print*, 'Use up to latest 13 (corresponding to 3 days) vit records data'
    DO n = vipmax-12,vipmax
      print*, 'Processing track location: Lon, Lat=', vitX(n), vitY(n)
!     if ( vRMAXa(n) < 0 ) then
!       vitR = 50.
!     else
!       vitR=MIN(MAX(vRMAXa(n), 25.0), 100.)
!     endif
      vitR = 25.
      print*, 'vitR=',vitR
      DO j = 1,jm
        DO i = 1,im
          ! Calculate distance from vitX(n), vitY(n)
          dlonlat = sqrt( mod((east_e(i,j)-vitX(n)),360.)**2 + mod((north_e(i,j)-vitY(n)),360.)**2 )
          dvkm = rearth*dlonlat*d2r/1000.
          dvr = dvkm/vitR/2.
          !print*, 'dlonlat,dvr,dvkm,vitR=',dlonlat,dvr,dvkm,vitR
          IF ( dvr > 3.0 ) THEN
            t(i,j,1:nl) = tmix(i,j,1:nl)
          ELSE IF ( dvr > 2.0 .AND. dvr <= 3.0 ) THEN
              t(i,j,1:nl) = (dvr-2.0)/1.0*tmix(i,j,1:nl) + (3.0-dvr)/1.0 * told(i,j,1:nl)
          ELSE
            !print*, 'Keep RTOFS T for i,j,east_e,north_e=',i,j,east_e(i,j),north_e(i,j)
            t(i,j,1:nl)=told(i,j,1:nl)
          END IF
        END DO
      END DO
      tmix=t
    END DO
  ELSE IF ( vipmax > 0 ) THEN
    ! R > 6*vitR:          Assimilate GFS SST 
    ! 4*vitR < R < 6*vitR: Merge GFS SST and RTOFS T
    ! R < 4*vitR:          Use RTOFS T
    print*, 'Track file exists, use RTOFS SST along the track'
    DO n = 1,vipmax
      print*, 'Processing track location: Lon, Lat=', vitX(n), vitY(n)
!     if ( vRMAXa(n) < 0 ) then
!       vitR = 50.
!     else
!       vitR=MIN(MAX(vRMAXa(n), 25.0), 100.)
!     endif
      vitR = 25.
      print*, 'vitR=',vitR
      DO j = 1,jm
        DO i = 1,im
          ! Calculate distance from vitX(n), vitY(n)
          dlonlat = sqrt( mod((east_e(i,j)-vitX(n)),360.)**2 + mod((north_e(i,j)-vitY(n)),360.)**2 )
          dvkm = rearth*dlonlat*d2r/1000.
          dvr = dvkm/vitR/2.
          !print*, 'dlonlat,dvr,dvkm,vitR=',dlonlat,dvr,dvkm,vitR
          IF ( dvr > 3.0 ) THEN
            t(i,j,1:nl) = tmix(i,j,1:nl)
          ELSE IF ( dvr > 2.0 .AND. dvr <= 3.0 ) THEN
              t(i,j,1:nl) = (dvr-2.0)/1.0*tmix(i,j,1:nl) + (3.0-dvr)/1.0 * told(i,j,1:nl)
          ELSE
            !print*, 'Keep RTOFS T for i,j,east_e,north_e=',i,j,east_e(i,j),north_e(i,j)
            t(i,j,1:nl)=told(i,j,1:nl)
          END IF
        END DO
      END DO
      tmix=t
    END DO
  ELSE
    ! In case vipmax <= 0, which means no available track info 
    print*, 'Use RTOFS SST around storm center'
    print*, 'Processing storm center: clon, clat=', clon, clat 
!   vitR=50.
    vitR=25.
    print*, 'vitR=',vitR
    DO j = 1,jm
      DO i = 1,im
        ! Calculate distance from clon, clat
        dlonlat = sqrt( mod((east_e(i,j)-clon),360.)**2 + mod((north_e(i,j)-clat),360.)**2 )
        dvkm = rearth*dlonlat*d2r/1000.
        dvr = dvkm/vitR/2.
        !print*, 'dlonlat,dvr,dvkm,vitR=',dlonlat,dvr,dvkm,vitR
        IF ( dvr > 3.0 ) THEN
          t(i,j,1:nl) = tmix(i,j,1:nl)
        ELSE IF ( dvr > 2.0 .AND. dvr <= 3.0) THEN
          t(i,j,1:nl) = (dvr-2.0)/1.0*tmix(i,j,1:nl) + (3.0-dvr)/1.0 * told(i,j,1:nl)
        ELSE
          !print*, 'Keep RTOFS T for i,j,east_e,north_e=',i,j,east_e(i,j),north_e(i,j)
          t(i,j,1:nl)=told(i,j,1:nl)
        END IF
      END DO
    END DO
  END IF

 END IF

! MPIPOM-TC temperature and salinity modification

  DO k = 1,nl
    DO j = 1,jm
       DO i = 1,im
         t(i,j,k) = t(i,j,k)*fsm(i,j)
         s(i,j,k) = s(i,j,k)*fsm(i,j)
         IF( zrtof(k) <=  h(i,j) .AND. fsm(i,j) == 1.0 ) THEN
           IF (t(i,j,k) == largetval .OR. s(i,j,k) == largetval) THEN
             t(i,j,k) = 0.
             s(i,j,k) = 0.
             uin(i,j,k) = 0.
             vin(i,j,k) = 0.
             PRINT*, 'Bprobl',  east_e(i,j),north_e(i,j),k, h(i,j),fsm(i,j)
          END IF
        END IF
      END DO
    END DO
  END DO

! (for zrtof > h replace largetval with neutral profile)

!  DO j = 1,jm
!    DO i = 1,im
!      DO k = 2,nl
!        IF ( zrtof(k) > h(i,j) .AND. fsm(i,j) == 1.0 ) THEN
!          IF (t(i,j,k) == largetval .OR. s(i,j,k) ==  largetval ) THEN
!            t(i,j,k) = t(i,j,k-1)
!            s(i,j,k) = s(i,j,k-1)
!          END IF
!        END IF
!      END DO
!    END DO
!  END DO


! MPIPOM-TC u and v modification

  DO k = 1,nl
    IF (ionedim.eq.1) THEN
      DO j = 1,jm
        DO i = 1,im
          u(i,j,k) = uin(i,j,k)*dum(i,j)
          v(i,j,k) = vin(i,j,k)*dvm(i,j)
        END DO
      END DO
    ELSE
      DO j = 1,jm
        u(1,j,k) = uin(1,j,k)*dum(1,j)
          DO i = 2,im
             u(i,j,k) = 0.5*(uin(i-1,j,k)+uin(i,j,k))*dum(i,j)
          END DO
      END DO
      DO i = 1,im
        v(i,1,k) = vin(i,1,k)*dvm(i,1)
        DO j = 2,jm
          v(i,j,k) = 0.5*(vin(i,j-1,k)+vin(i,j,k))*dvm(i,j)
        END DO
      END DO
    END IF
  END DO

! MPIPOM-TC el modification

 DO j = 1,jm
   DO i = 1,im
     el(i,j) = el(i,j)*fsm(i,j)
   END DO
 END DO

! MPIPOM-TC tclim and sclim calcuation

   DO k = 1,nl
     taa(k) = 0.
     saa(k) = 0.
     area(k) = 0.
     DO j = 1,jm
       DO i = 1,im
         IF (t(i,j,k) /= 0.) THEN 
           taa(k) = taa(k)+t(i,j,k)
           saa(k) = saa(k)+s(i,j,k)
           area(k) = area(k)+1.
         END IF
       END DO
     END DO
     taa(k) = taa(k)/area(k)
     saa(k) = saa(k)/area(k)
   END DO

   DO k = 1,nl
     DO j = 1,jm
       DO i = 1,im
          tclim(i,j,k) = taa(k)
          sclim(i,j,k) = saa(k)
       END DO
     END DO
   END DO

! Prepare to create MPIPOM-TC input files

  WRITE(6,*) TRIM(ADJUSTL(runid))
  
  CALL outncf

END PROGRAM

